/*
 * CHECK-UTILS
 *
 * CVS ID Tag
 * $Id: checkutils.c,v 1.17 2006/06/26 06:22:53 oops Exp $
 */
#include <checkutils.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#ifdef HAVE_REGEX_H
#	include <regex.h>
#endif
#include <ctype.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <assert.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/types.h>
#include <fcntl.h>
#include <errno.h>
#ifdef HAVE_ICONV_H
#include <iconv.h>
#endif

#include <sys/fcntl.h>
#include <netinet/tcp.h>

void i18n_print(void)
{
#ifdef ENABLE_NLS
	setlocale (LC_CTYPE, "");
	setlocale (LC_MESSAGES, "");
	bindtextdomain("check-utils", LANGDIR);
	textdomain("check-utils");
#endif
}

extern int verbose;

unsigned int checkReg(unsigned char *str, unsigned char *regex_o) {
	regex_t reg;

	if (regcomp(&reg, regex_o, REG_EXTENDED|REG_ICASE) != 0) {
		fprintf(stderr,_(" Problem REGEX compile '%s'\n"), regex_o);
		return 0;
	}

	if (regexec(&reg, str, 0, NULL, 0) == 0) {
		regfree(&reg);
		return 1;
	} else {
		regfree(&reg);
		return 0;
	}
}

long long charToint (char *str) {
	int len = strlen (str), i = 0;
	int num = len - 1, j = 0;
	long long val = 0, res = 0;
	int minus = 0, start = 0;

	if ( str[0] == '-' ) {
		minus = 1;
		start = 1;
		num--;
	}

	for (i=start; i < len; i++) {
		switch (str[i]) {
			case '1' :
				val = 1;
				break;
			case '2' :
				val = 2;
				break;
			case '3' :
				val = 3;
				break;
			case '4' :
				val = 4;
				break;
			case '5' :
				val = 5;
				break;
			case '6' :
				val = 6;
				break;
			case '7' :
				val = 7;
				break;
			case '8' :
				val = 8;
				break;
			case '9' :
				val = 9;
				break;
			default :
				val = 0;
		}

		for (j=0; j < num; j++) {
			val *= 10;
		}
		num--;

		res += val;
	}

	if ( minus == 1 )
		res *= -1;

	return res;
}

double microtime (void) {
	struct timeval tp;
	long sec = 0L;
	double ret = 0;
	double msec = 0.0;

	if ( gettimeofday ((struct timeval *) &tp, '\0') == 0) {
		msec = (double) (tp.tv_usec / 1000000.00);
		sec = tp.tv_sec;

		if (msec >= 1.0) msec -= (long) msec;
		ret = sec + msec;
	}

	return ret;
}

/*
 * This code follows PHP License
 * sub  => TRUE:    FALSE: NULL
 * uint => TRUE: bit          FALSE: byte
 */
void human_size (double size_o, int sub, int unit, char * ret) {
	float res;
	unsigned char sunit[6], ssunit, re_unit[3];
	char * BYTE_C, bytes[1024];
	char units[] = { 'K', 'M', 'G', 'T', 'P' };
	int i = -1;
	
	memset (sunit, '\0', sizeof(sunit));
	memset (re_unit, '\0', sizeof(re_unit));

	if (unit != 1) {
		strcpy (sunit, "Bytes");
		ssunit = 'B';
	} else {
		strcpy (sunit, "Bits");
		ssunit = 'b';
	}

	BYTE_C = (char *) numberFormat (size_o, 0, '.', ',', 0);
	strcpy (bytes, BYTE_C);
	free(BYTE_C);

	if (size_o < 1024) {
		sprintf (ret, "%s %s", bytes, sunit);
	} else {
		while ( size_o > 1024 ) {
			size_o /= 1024;
			i++;
		}

		res = size_o;

		memset ( re_unit, units[i], 1);
		memset (re_unit + 1, ssunit, 1);

		if (sub)
			sprintf(ret, "%.2f %s (%s %s)", res, re_unit, bytes, sunit);
		else
			sprintf(ret, "%.2f %s", res, re_unit);

	}
}

/*
 * This code follows PHP License
 */
char * numberFormat (double d, int dec, char dec_point, char thousand_sep, int print) {
	char *tmpbuf, *resbuf;
	char *src, *tgt;  /* source, target */
	int tmplen, reslen = 0;
	int count = 0;
	int is_negative = 0;

	if (d < 0) {
		is_negative = 1;
		d = -d;
	}
	dec = MAX(0, dec);
	tmpbuf = (char *) malloc (1024);

	tmplen = sprintf (tmpbuf, "%.*f", dec, d);

	if ( !isdigit ((int) tmpbuf[0]) ) {
		return tmpbuf;
	}

	if ( dec )
		reslen = dec + 1 + (tmplen - dec - 1) + ((thousand_sep) ? (tmplen - 1 - dec - 1) / 3 : 0);
	else
		reslen = tmplen + ((thousand_sep) ? (tmplen - 1) / 3 : 0);

	if (is_negative) 
		reslen++;

	resbuf = (char *) malloc (reslen + 1);

	src = tmpbuf + tmplen - 1;
	tgt = resbuf + reslen;
	*tgt-- = 0;

	if ( dec ) {
		while (isdigit ((int) *src)) {
			*tgt-- = *src--;
		}

		*tgt-- = dec_point;  /* copy that dot */
		src--;
	}

	while( src >= tmpbuf ) {
		*tgt-- = *src--;
		if (thousand_sep && (++count % 3) == 0 && src >= tmpbuf) {
			*tgt-- = thousand_sep;
		}
	}
	if ( is_negative ) {
		*tgt-- = '-';
	}

	free(tmpbuf);

	if (print) {
		printf ("%s", resbuf);
		free (resbuf);
		return NULL;
	} else {
		return resbuf;
	}
}

void print_ansi (int color, int noansi) {
	int ansi;

	switch ( color ) {
		case GRAY :     ansi = 30; break;
		case BGRAY :    ansi = 40; break;
		case RED :      ansi = 31; break;
		case BRED :     ansi = 41; break;
		case GREEN :    ansi = 32; break;
		case BGREEN :   ansi = 42; break;
		case YELLOW :   ansi = 33; break;
		case BYELLOW :  ansi = 43; break;
		case BLUE :     ansi = 34; break;
		case BBLUE :    ansi = 44; break;
		case MAGENTA :  ansi = 35; break;
		case BMAGENTA : ansi = 45; break;
		case CYAN :     ansi = 36; break;
		case BCYAN :    ansi = 46; break;
		case WHITE :    ansi = 37; break;
		case BWHITE :   ansi = 47; break;
		default :       ansi = 0;
	}

	if ( ! noansi ) {
		if ( color == ENDANSI )
			printf ("[7;0m");
		else
			printf ("[1;%dm", ansi);
	}
}

char * get_ansi (int color, int noansi) {
	int ansi;
	static char ret[32] = { 0, };

	if ( noansi )
		return "";

	switch ( color ) {
		case GRAY     : ansi = 30; break;
		case BGRAY    : ansi = 40; break;
		case RED      : ansi = 31; break;
		case BRED     : ansi = 41; break;
		case GREEN    : ansi = 32; break;
		case BGREEN   : ansi = 42; break;
		case YELLOW   : ansi = 33; break;
		case BYELLOW  : ansi = 43; break;
		case BLUE     : ansi = 34; break;
		case BBLUE    : ansi = 44; break;
		case MAGENTA  : ansi = 35; break;
		case BMAGENTA : ansi = 45; break;
		case CYAN     : ansi = 36; break;
		case BCYAN    : ansi = 46; break;
		case WHITE    : ansi = 37; break;
		case BWHITE   : ansi = 47; break;
		default       : ansi = 0;
	}

	if ( color == ENDANSI )
		return "\e[7;0m";
	else {
		sprintf (ret, "\e[1;%dm", ansi);
		return ret;
	}
}

char * trim(char *string)
{
	int l = 0, i = 0, ret = 0; 

	if (string != NULL) {
		if ( (l = strlen(string)) > 0) {
			for (i = 0; i < l; i++) { 
				if ( !isspace(string[i]) ) {
					ret = i;
					break;
				}
			}

			for (i=l-1; i>-1; i--) {
				if ( !isspace(string[i]) ) {
					string[i+1] = '\0';
					break;
				}
			}

			return string + ret;
		}
	}

	return "";
}

char * readfile (char *filename) {
	FILE *fp;
	size_t len = 0, tmplen = 0;
	char *text = NULL, tmptext[FILEBUFS];

	if ((fp = fopen(filename, "rb")) == NULL) {
		fprintf (stderr, _("Can't open %s in read mode"), filename);
		return NULL;
	}

	while ( (len = fread (tmptext, sizeof(char), FILEBUFS, fp)) > 0 ) {
		if (tmplen == 0)
			text = realloc (text, sizeof (char) * (tmplen + len + 2));
		else
			text = malloc (sizeof (char) * len + 2);

		memmove (text + tmplen, tmptext, len);
		tmplen += len;
		memset (tmptext, '\0', sizeof (tmptext));
	}

	text[tmplen] = '\0';
	fclose (fp);

	return text;
}

char * getcmds ( char * cmd ) {
	FILE * pipe = NULL;
	char readv[READBUFS] = { 0, }, *buf = NULL;
	int readi = 0, buflen = 0;

	memset ( readv, 0, READBUFS );
	buf = malloc (sizeof (char));

	pipe = popen ( cmd, "r" );

	if ( pipe == NULL ) {
		return NULL;
	}

	while ( ( readi = fread ( readv, sizeof (char), READBUFS - 1, pipe ) ) > 0 ) {
		readv[readi] = 0;
		buf = realloc ( buf, sizeof (char) * ( buflen + readi + 2 ) );
		memmove ( buf + buflen, readv, readi );
		memset ( readv, 0, READBUFS );
		buflen += readi;
		memset (buf + buflen, 0, 1);
	}

	pclose (pipe);

	if ( buflen < 1 )
		return NULL;

	return buf;
}


int string_utf_iconv (char *dest, char *src, int len, int type) {
	// int i;
	// int orig_len = len;
	// int utf16_len = len * 2 + 32;

#if defined(HAVE_LIBICONV) || defined(__CYGWIN__) || defined(__FreeBSD__)
	const
#endif
	char* inbuf_p = src;
	char* outbuf_p = dest;

	size_t il = strlen(src) + 1;
	size_t ol = 1000;

	iconv_t cd;

	if ( type == 16 )
		cd = iconv_open ("UTF-16LE", "EUC-KR");
	else
		cd = iconv_open ("UTF-8", "EUC-KR");

	if ( !cd )
		return 1;

	if ( iconv (cd, &inbuf_p, &il, &outbuf_p, &ol) == -1 || il != 0)
		return 1;

	/*
	fprintf (stderr, "==> %s\n", src);
	for (i = 0; i < orig_len * 2; i++)
		printf ("[%02x] %c\n", dest[i], dest[i]);
	*/

	iconv_close(cd);
	return 0;
}

int is_anychar (char c) {
	/*
	 * 0-9 => 45-57
	 * a-z => 97-122
	 * A-Z => 65-90
	 * /   => 47
	 * .   => 46
	 * -   => 45
	 * =   => 61
	 */
	if ( c > 45 && c < 58 )
		return 0;

	if ( c == 61 )
		return 0;

	if ( c > 64 && c < 91 )
		return 0;

	if ( c > 96 && c < 123 )
		return 0;

	return 1;
}

void URLEncode (char *dst, char *src) {
	int   i = 0, j = 0;
	char  buf[5] = { 0, };

	if ( src == NULL || dst == NULL )
		return;

	while ( src[i] != 0 ) {
		if ( ! is_anychar (src[i]) )
			sprintf (buf, "%c", src[i]);
		else
			sprintf (buf, "%%%02x", src[i] & 0xff);

		memcpy (dst + j, buf, strlen(buf));
		j += strlen (buf);
		i++;
	}
}

int nonblock_func (int socket, int nonblock) {
	int flags;

	flags = fcntl (socket, F_GETFL, 0);

	if ( TRUE == nonblock )
		flags |= O_NONBLOCK;
	else
		flags &= ~O_NONBLOCK;

  return fcntl (socket, F_SETFL, flags);
}

/*
 * This code get check-utils package from BPL License
 * See Also http://cvs.oops.org/index.cgi/check-utils/BPL
 */
int HOSTconnect (char * addr, int timeout, int port, int blockon) {
	int                  s = -1;
	struct sockaddr_in   sa;
	struct sockaddr_in   la;
	struct hostent      *hp;
	int                  result = 0;
	struct timeval       timeouts;
	fd_set               c_fds, r_fds;
	int                  sval;
	socklen_t            slen;

	/* DNS lookup */
	if ((hp = gethostbyname(addr)) == NULL) {
		fprintf(stderr, _("Host name lookup failure.\n"));
		return -1;
	}

	timeouts.tv_sec = timeout;
	timeouts.tv_usec = 0;

	setsockopt(s, SOL_SOCKET, SO_SNDTIMEO, &timeouts, sizeof(timeouts));
	setsockopt(s, SOL_SOCKET, SO_RCVTIMEO, &timeouts, sizeof(timeouts));
	setsockopt(s, SOL_SOCKET, SO_KEEPALIVE, NULL, 0);
	setsockopt(s, SOL_SOCKET, SO_REUSEADDR, NULL, 0);
	setsockopt(s, IPPROTO_TCP, TCP_NODELAY, NULL, 0);

	/* fill socket structure */

	bcopy ((char *) hp->h_addr, (char *) &sa.sin_addr, hp->h_length);
	sa.sin_family = hp->h_addrtype;
	/*sa.sin_port = 0x5000;*/ /* http port (80 => 50 Hex, switch Hi-/Lo-Word ) */

	sa.sin_port = htons(port) ; /* be2me_16(1755);  mms port5 */

	if (verbose) printf (_("port : %08x, %d\n"), sa.sin_port, ntohs (sa.sin_port));

	/* open socket */

	if ((s = socket(hp->h_addrtype, SOCK_STREAM, 0))<0) {
		fprintf (stderr, _("can't create socket\n"));
		return -1;
	}

	if (verbose) printf (_("socket open\n"));

	/* local socket */
	la.sin_family = hp->h_addrtype;
	la.sin_addr.s_addr = htonl (INADDR_ANY);
	la.sin_port = htons (0);

	/* local bind */
	if ( bind (s, (struct sockaddr *) &la, sizeof (la)) < 0 ) {
		close (s);
		return -1;
	}

	/* make it nonblocking funtion connection */
	if ( nonblock_func (s, TRUE) == -1 ) {
		close (s);
		return -1;
	}

	/* try to connect */

	if ( connect (s, (struct sockaddr *)&sa, sizeof sa) < 0 && errno == EINPROGRESS) {
		/* connected nonblockingly. wait until timeout */
		timeouts.tv_sec = timeout;
		timeouts.tv_usec = 0;

		FD_ZERO (&c_fds);
		FD_ZERO (&r_fds);
		FD_SET (s, &c_fds);
		FD_SET (s, &r_fds);

		/* wait untim something happens */
		if ( (result = select ( s + 1, &r_fds, &c_fds, NULL, &timeouts )) != 1 ) {
			/* failed connect () */
			fprintf (stderr, _("==> ERROR: connect failed code %d\n"), result);
			close (s);
			return -1;
		}

		if ( FD_ISSET (s, &r_fds) || FD_ISSET (s, &c_fds) ) {
			if ( FD_ISSET (s, &r_fds) && FD_ISSET (s, &c_fds) ) {
				slen = sizeof (sval);

				if ( getsockopt (s, SOL_SOCKET, SO_ERROR, &sval, &slen) < 0 ) {
					fprintf (stderr, _("==> ERROR: %s\n"), strerror (ETIMEDOUT));
					close (s);

					return -1;
				}

				if ( sval != 0 ) {
					fprintf (stderr, _("==> ERROR: %s\n"), strerror (sval));
					close (s);
					return -1;
				}
			}
		}
	}

	if (verbose) printf (_("connected\n"));

	if ( blockon ) {
		/* return blocking mode */
		if ( nonblock_func (s, FALSE) == -1 ) {
			close (s);
			return -1;
		}
	}

  return s;
}

int chunked_hex2int (char *str) {
	char chunkedstr[8] = { 0, };

	sprintf (chunkedstr, "0x%s", str);

	return strtol (chunkedstr, (char **) NULL, 16);
}


char * field_string (char *text, int col) {
	const char delimiters[] = " \n";
	char *buf;
	char *token, *btoken;
	static char str[1024] = { 0, };
	int i;

	buf = (char *) malloc (sizeof (char) * (strlen (text) + 1));
	strcpy (buf, text);

	token = (char *) strtok_r (buf, delimiters, &btoken);

	i = 1;
	while ( token != NULL ) {
		if ( i == col ) {
			strcpy (str, trim(token));
			break;
		}
		token = (char *) strtok_r (NULL, delimiters, &btoken);
		i++;
	}

	free (buf);

	return str;
}

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
