/*
 * CHECK-UTILS : chkbasdwidth
 * chkbandwidth is check current bandwidth on server
 *
 * $Id: chkbandwidth.c,v 1.16 2006/05/29 12:53:36 oops Exp $
 */

#include <checkutils.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#include <ctype.h>
#include <signal.h>
#ifdef HAVE_NCURSES_H
#include <ncurses.h>
#else
#include <curses.h>
#endif
#ifndef __CYGWIN__
#include <term.h>
#else
#include <ncurses/term.h>
#endif
#include <sys/ioctl.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <chkbandwidth.h>
#include <cursesset.h>

int verbose = 0;
int prevlen = 0;

int curses_open = 0;
int change_window_size = 0;

struct scrsize sc = { 0, 0 };

static void window_handler (int s) {
	change_window_size = 1;
}

struct interface ifs = {
	{NULL, },
	{NULL, },
	{NULL, },
	-1,
	{0, },
	{0, }
};

int main (int argc, char *argv[]) {
	int opt, optchklen = 0;

	/* option variables */
	struct configs conf = { 2, 1, 0, { 0, }, { 0, }, 0 };
	short firsttime = 1;
	short chgkey    = 0;

	struct packets po = { 0, 0 };
	//struct interface ifs = { {NULL, }, {NULL, }, {NULL, }, -1, {0, }, {0, } };

	i18n_print ();

#ifdef HAVE_GETOPT_LONG
    while ( (opt = getopt_long (argc, argv, "ci:hs:u:v?", long_options, (int *) 0)) != EOF ) {
#else
	while ( (opt = getopt (argc, argv, "ci:hs:u:v?")) != EOF ) {
#endif
		optchklen = 0;

		switch (opt) {
			 /* verbose option */
			case 'v' :
				verbose = 1;
				break;
			/* interactive mode */
			case 'c' :
				conf.nocurses = 1;
				break;
			/* interface option */
			case 'i' :
				if ( strlen (optarg) > 7 ) {
					fprintf (stderr, _("Interface option is to long\n"));
					return 1;
				}

				strcpy (conf.iface, optarg);
				break;
			/* second option */
			case 's' :
				conf.second = charToint (optarg);
				break;
			/* unit option */
			case 'u' :
				if ( strcmp (optarg, "byte") && strcmp (optarg, "bit") ) {
					fprintf (stderr, _("unit option must be byte or bit\n"));
					return 1;
				}

				strcpy (conf.unit, optarg);
				break;
			/* hele message */
			case 'h' :
			default :
				conf.usages = 1;
		}
	}
	
    if ( argc != optind )
		conf.usages = 1;

	if ( strlen (conf.unit) < 1 )
		strcpy (conf.unit, "bit");
	conf.unitarg = ( ! strcmp (conf.unit, "bit") ) ? 1: 0;

	if (strlen (conf.iface) < 1)
		strcpy (conf.iface, "eth0");

	if ( conf.usages ) {
		usage (argv[0]);
		return 1;
	}

	/*
	 * not check on cygwin
	 */
	if ( iface_info (&ifs, conf.iface) == -1 ) {
		fprintf (stderr, _("No device %s on this system\n"), conf.iface);
		return 1;
	}

	/* for debugging ..
	{
		int i;
		for ( i=0;i<4;i++ ) {
			printf ("DEVEICE : %s\n", ifs.ifname[i]);
			printf ("IN      : %s\n", ifs.in[i]);
			printf ("OUT     : %s\n", ifs.out[i]);
		}
		printf ("device %d => %s\n", ifs.dev, ifs.ifname[ifs.dev]);
		exit (1);
	}
	*/

	cb_get_terminal_size ();

	if ( conf.nocurses )
		goto nocurses;

	signal (SIGINT,   cb_exit);
	signal (SIGTERM,  cb_exit);
	signal (SIGHUP,   cb_exit);
	signal (SIGQUIT,  cb_exit);
	signal (SIGCONT,  window_handler);
	signal (SIGWINCH, window_handler);

	/* init curses */
	cb_init ();
	bkgd (COLOR_PAIR(COLOR_DEF));

	while ( 1 ) {
		char banner[256] = { 0, };
		time_t _t = time (NULL);
		char *t = ctime(&_t);
		size_t tlen = strlen (t);
		short row = 0;
		char key;
		fd_set fs;
		long file_flags;

		struct packets p = { 0, 0 };

		FD_ZERO (&fs);
		FD_SET(STDIN_FILENO, &fs);
		file_flags = fcntl(STDIN_FILENO, F_GETFL);

		if ( file_flags == -1 )
			file_flags = 0;

		fcntl(STDIN_FILENO, F_SETFL, O_NONBLOCK|file_flags);

		if ( cb_getch(&key) > 0 ) {
			fcntl(STDIN_FILENO, F_SETFL, file_flags);

			switch (key) {
				case 'b' :
					conf.unitarg = conf.unitarg ^ 1;
					break;
				case 'v' :
					verbose = verbose ^ 1;
					break;
				case 'q' :
					goto cursesexit;
					break;
			}
			chgkey = 1;
		} else
			fcntl(STDIN_FILENO, F_SETFL, file_flags);

		if ( change_window_size ) {
			cb_get_terminal_size ();
			resizeterm (sc.height, sc.width);
			clear ();
			/* reinit windows size */
			change_window_size = 0;
		} else {
			if ( chgkey > 0 ) {
				clear ();
				chgkey = 0;
			}
		}
			

		/* print banner */
		sprintf (banner, " CHECK-UTILS [%s] v%s ", argv[0], PACKAGE_VERSION);
		attron (COLOR_PAIR(COLOR_BANNER));
		attron (A_BOLD);
		mvaddstr (row, 0, banner);
		attroff (COLOR_PAIR(COLOR_BANNER));

		if ( strlen (banner) > (size_t) (sc.width - tlen - 1) )
			mvaddstr (row, sc.width - tlen - 4, "..  ");
		mvaddstr (row++, sc.width - tlen, t);
		attroff(A_BOLD);

		/* print result */
		attron (A_BOLD);
		mvprintw (++row, 2, _("* %s information :"), ifs.ifname[ifs.dev]);
		attroff (A_BOLD);

		iface_info (&ifs, NULL);

		if ( firsttime ) {
			po.in  = charToint (ifs.in[ifs.dev]);
			po.out = charToint (ifs.out[ifs.dev]);

			attron (A_BOLD);
			attron (COLOR_PAIR (COLOR_INOUT));
			mvprintw (++row, 2, _("Measuring now.. wait %d seconds"), conf.second);
			attroff (COLOR_PAIR (COLOR_INOUT));
			attroff (A_BOLD);
			chgkey = 1;
		} else {
			p.in = charToint (ifs.in[ifs.dev]);
			p.out = charToint (ifs.out[ifs.dev]);

			memset (ifs.sin, 0, sizeof (ifs.sin));
			memset (ifs.sout, 0, sizeof (ifs.sout));

			if ( conf.unitarg ) {
				human_size ( ((p.in - po.in) * 8) / conf.second, 0, conf.unitarg, ifs.sin);
				human_size ( ((p.out - po.out) * 8) / conf.second, 0, conf.unitarg, ifs.sout);
			} else {
				human_size ( (p.in - po.in) / conf.second, 0, conf.unitarg, ifs.sin);
				human_size ( (p.out - po.out) / conf.second, 0, conf.unitarg, ifs.sout);
			}

			attron (A_BOLD);
			attron (COLOR_PAIR (COLOR_INOUT));
			mvaddstr (++row, 2, _("[ I N ]"));
			attroff (COLOR_PAIR (COLOR_INOUT));
			attroff (A_BOLD);

			if ( verbose )
				mvprintw (row, 15, "%15lld %15lld %10s/s (%ld)%15c",
						po.in, p.in, ifs.sin, p.in - po.in, 0);
			else
				mvprintw (row, 20, "%20s/s", ifs.sin);

			attron (A_BOLD);
			attron (COLOR_PAIR (COLOR_INOUT));
			mvaddstr (++row, 2, _("[ OUT ]"));
			attroff (COLOR_PAIR (COLOR_INOUT));
			attroff (A_BOLD);

			if ( verbose )
				mvprintw (row, 15, "%15lld %15lld %10s/s (%ld)%15c",
					   	po.out, p.out, ifs.sout, p.out - po.out, 0);
			else
				mvprintw (row, 20, "%20s/s", ifs.sout);

			po.in  = p.in;
			po.out = p.out;

			row++;

			mvprintw (++row, 1, _("This program checked current network bandwidth on server."));
			mvprintw (++row, 1, _("Options :"));
			mvprintw (++row, 3, _("b: switched bandwidth unit (byte or bit)"));
			mvprintw (++row, 3, _("v: switched verbose mode"));
			mvprintw (++row, 6, _("print order as <old current value value_bit>"));
			mvprintw (++row, 3, _("q: quit this program after %d seconds"), conf.second);
			row++;
			mvprintw (++row, 1, _("Switch : "));
		}

		firsttime = 0;
		refresh ();
		usleep (conf.second * 1000000);
	}

cursesexit:
	if ( curses_open )
		endwin ();

	goto toexit;

nocurses:
	{
		struct packets p = { 0, 0 };

		iface_info (&ifs, conf.iface);
		po.in  = charToint (ifs.in[ifs.dev]);
		po.out = charToint (ifs.out[ifs.dev]);

		usleep (conf.second * 1000000);

		iface_info (&ifs, NULL);
		p.in  = charToint (ifs.in[ifs.dev]);
		p.out = charToint (ifs.out[ifs.dev]);

		memset (ifs.sin, 0, sizeof (ifs.sin));
		memset (ifs.sout, 0, sizeof (ifs.sout));

		if ( conf.unitarg ) {
			human_size ( ((p.in - po.in) * 8) / conf.second, 0, conf.unitarg, ifs.sin);
			human_size ( ((p.out - po.out) * 8) / conf.second, 0, conf.unitarg, ifs.sout);
		} else {
			human_size ( (p.in - po.in) / conf.second, 0, conf.unitarg, ifs.sin);
			human_size ( (p.out - po.out) / conf.second, 0, conf.unitarg, ifs.sout);
		}

		fflush (stdout);

		if ( verbose )
			printf ("%s %lld %lld %s    %s %lld %lld %s\n",
					_("[ I N ]"), p.in, po.in, ifs.sin,
					_("[ OUT ]"), p.out, po.out, ifs.sout);
		else
			printf ("%s: %s %s %s %s\n", ifs.ifname[ifs.dev], _("[ I N ]"), ifs.sin, _("[ OUT ]"), ifs.sout);
	}

toexit:
	interface_free (&ifs);

	return 0;
}

#if defined(__linux__)
#include "os/linux/chkbandwidth_os.c"
#elif defined(__CYGWIN__)
#include "os/cygwin/chkbandwidth_os.c"
#elif defined (__FreeBSD__)
#include "os/freebsd/chkbandwidth_os.c"
#endif

void usage (char *prog) {
	print_ansi (BLUE, 0);
	printf ("CHECK-UTILS [%s] v%s : ", prog, PACKAGE_VERSION);
	print_ansi (ENDANSI, 0);
	printf (_("Check current bandwidth on server\n\n"));

	fprintf (stderr, _("Usage : %s [Option[:Value]]\n"), prog);
#ifdef HAVE_GETOPT_LONG
	fprintf (stderr, _("        -i, --interface=dev  Ethernet interface [ default: eth0 ]\n"));
	fprintf (stderr, _("        -u, --unit=byte/bit  unit of measurement bit/byte [ default: bit ]\n"));
	fprintf (stderr, _("        -s, --seconds=sec    interval time of measurement [ default: 2sec ]\n"));
	fprintf (stderr, _("        -c, --interactive    interactive mode\n"));
	fprintf (stderr, _("        -v, --verbose        verbose option\n"));
	fprintf (stderr, _("        -h, --help           This (help) message\n"));
#else
	fprintf (stderr, _("        -i dev       Ethernet interface [ default: eth0 ]\n"));
	fprintf (stderr, _("        -u byte/bit  unit of measurement bit/byte [ default: bit ]\n"));
	fprintf (stderr, _("        -s sec       interval time of measurement [ default: 2sec ]\n"));
	fprintf (stderr, _("        -c           interactive mode\n"));
	fprintf (stderr, _("        -v           verbose option\n"));
	fprintf (stderr, _("        -h           This (help) message\n"));
#endif
}

void interface_free (struct interface *p) {
	int i = 0;

	while ( p->ifname[i] != NULL ) {
		if ( i > DEVICEMAX )
			break;

		free (p->ifname[i]);
		free (p->in[i]);
		free (p->out[i]);
		i++;
	}
}

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
