/*
 * CHECK-UTILS : chkdownload
 * chkdownload is check download bandwidth with http protocol
 *
 * $Id: chkdownload.c,v 1.30 2006/04/24 01:11:27 oops Exp $
 */

#include <checkutils.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/fcntl.h>
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <time.h>
#include <chkdownload.h>
#include <http.h>

int verbose = 0;

struct tm *loctime;
struct stat filestat;

char loginit[1024];
char logmsg[1024];
int return_code, sock_code;
long return_length, headlen;

char save_value[SAVEBUF+1];
size_t save_len = 0;

int main(int argc ,char *argv[])
{
	FILE *fd = NULL;
	int opt, optchklen = 0, rmbuflen = 0;
	int sock, retval, savechk = 0, headsum = 0;
	long long filesize = 0;
	time_t now = time (NULL);
	double start_time, end_time, exec_time;
	double bytesecs, bitsecs;
	char h_bytesecs[32], h_bitsecs[32];
	fd_set w_fds;

	char cmd[2048], rcmd[2048], mark = '-';
	char buffer[READBUFS], timestr[64];

	/* option variables */
	long fsize = 0;
	int verbose = 0, noansi = 0;
	int rtimeout = 30, ctimeout = 30, httpver = 0;
	char hostname[256], addr[256], uri[1024], savefile[256];
	char agent[64];
	int port = 80;

	/* tmp variabls */
	char _host[256] = { 0, };

	i18n_print();

	memset (hostname, '\0', sizeof (hostname));
	memset (addr, '\0', sizeof (addr));
	memset (uri, '\0', sizeof (uri));
	memset (savefile, '\0', sizeof (savefile));
	memset (agent, '\0', sizeof (agent));
	memset (buffer, '\0', sizeof (buffer));
	memset (loginit, '\0', sizeof (loginit));

	loctime = localtime (&now);
	strftime(timestr, sizeof (timestr), "%Y.%m.%d %H:%M:%S", loctime);

#ifdef HAVE_GETOPT_LONG
	while ( (opt = getopt_long (argc, argv, "nh?H:a:u:S:f:r:c:A:p:1v", long_options, (int *) 0)) != EOF ) {
#else
	while ( (opt = getopt (argc, argv, "nh?H:a:u:S:f:r:c:A:p:1v")) != EOF ) {
#endif
		optchklen = 0;

		switch (opt) {
			// verbose option
			case 'v' :
				verbose = 1;
				break;
			// ansi option
			case 'n' :
				noansi = 1;
				break;
			// Host Header value
			case 'H' :
				if ( checkReg (optarg, "^http://") )
					optchklen = 7;

				if ( strlen (optarg + optchklen) > 255 ) {
					fprintf (stderr, _("%s Host header is too long\n"), timestr);
					return 1;
				}

				strcpy (hostname, optarg + optchklen);
				break;
			// connected address
			case 'a' :
				if ( checkReg (optarg, "^http://") )
					optchklen = 7;

				if ( strlen (optarg + optchklen) > 255 ) {
					fprintf (stderr, _("%s Address is too long\n"), timestr);
					return 1;
				}

				strcpy (addr, optarg + optchklen);
				break;
			// agent option
			case 'A' :
				if ( strlen (optarg) > 63 ) {
					fprintf (stderr, _("%s Agent name is too long\n"), timestr);
					return 1;
				}

				strcpy (agent, optarg);
				break;
			// uri option
			case 'u' :
				if ( checkReg (optarg, "^/") )
					optchklen = 1;

				if ( strlen (optarg + optchklen) > 1023 ) {
					fprintf (stderr, _("%s URI is too long\n"), timestr);
					return 1;
				}

				strcpy (uri, optarg + optchklen);
				break;
			// save file name
			case 'S' :
				if ( strlen (optarg) > 255 ) {
					fprintf (stderr, _("%s Save file path is too long\n"), timestr);
				}

				strcpy (savefile, optarg);
				memset (savefile + strlen(optarg), '\0', 1);
				break;
			case 'f' :
				fsize = charToint(optarg);
				break;
			case 'c' :
				ctimeout = charToint (optarg);
				break;
			case 'r' :
				rtimeout = charToint (optarg);
				break;
			case 'p' :
				port = charToint (optarg);
				break;
			case '1' :
				httpver = 1;
				break;
			case '0' :
				break;
			case 'h' :
			default :
				usage (argv[0]);
				return 1;
		}
	}

	if ( argc != optind || argc == 1 ) {
		usage (argv[0]);
		return 1;
	}

	if ( strlen (agent) < 1 ) {
		sprintf (agent, "CheckDownload %s", PACKAGE_VERSION);
	}

	if ( strlen (savefile) > 1 ) {
		savechk = 1;
	}

	if ( verbose ) {
		setbuf (stdout, NULL);
		printf ("\n");
		print_ansi (BLUE, noansi);
		printf ("CHECK-UTILS [%s] v%s : ", argv[0], PACKAGE_VERSION);
		print_ansi (ENDANSI, noansi);
		printf (_("check bandwidth with http download\n"));
		printf (_("     Copyright follows BPL License <http://devel.oops.org/document/bpl>\n"));
		printf (_("     JoungKyun Kim <http://www.oops.org>\n\n"));

		print_ansi (GREEN, noansi);
		printf (_("[ OPTIONS ]\n"));
		printf ("-------------------------------------------------------------------------------");
		print_ansi (ENDANSI, noansi);
		printf ("\n");
		printf ("VERBOSE     : %d\n", verbose);
		printf ("NOANSI      : %d\n", noansi);
		printf ("HEADER      : %s\n", hostname);
		printf ("ADDRESS     : %s\n", addr);
		printf ("URI         : %s\n", uri);
		printf ("AGENT       : %s\n", agent);
		printf ("SAVEFILE    : %s\n", savefile);
		printf ("FILESIZE    : %ld\n", fsize);
		printf ("CON_TIMEOUT : %d\n", ctimeout);
		printf ("REV_TIMEOUT : %d\n", rtimeout);
		printf ("\n");
	}

	if ( strlen (addr) < 4 ) {
		fprintf (stderr, _("%s Don't exist connect address with -a option\n"), timestr);
		return 1;
	}

	if ( ctimeout < 1 ) ctimeout = 5;
	if ( rtimeout < 1 ) rtimeout = 5;

	memset (cmd, '\0', sizeof(cmd));
	memset (rcmd, '\0', sizeof(rcmd));

	strcpy (_host, ( strlen (hostname) > 1 ) ? hostname : addr);
	/* with HTTP 1.1 */
	if ( httpver == 1 ) {
		sprintf (rcmd, "/%s HTTP/1.1\r\n"
					   "Host: %s\r\n"
					   "Accept: */*\r\n"
					   "Connection: close\r\n"
					   "User-Agent: %s\r\n\r\n",
				 uri, _host, agent);
	/* with HTTP 1.0 */
	} else {
		sprintf (rcmd, "http://%s/%s HTTP/1.0\r\n"
					   "Accept: */*\r\n"
					   "User-Agent: %s\r\n\r\n", _host, uri, agent);
	}

	sprintf (loginit, "%s http://%s/%s \"Host: %s\"", timestr,
			addr, uri, strlen (hostname) > 1 ? hostname : addr);

	sprintf (cmd, "GET %s", rcmd);

	if ( verbose ) {
		print_ansi (GREEN, noansi);
		printf (_("[ Execution Information ]\n"));
		printf ("-------------------------------------------------------------------------------");
		print_ansi (ENDANSI, noansi);
		printf ("\n");
		printf (_("Connection  : %s\n"), addr);
		
		/* with HTTP 1.1 */
		if ( httpver == 1 ) {
			printf (_("COMMAND     : GET|HEAD /%s HTTP/1.1\\r\\n\n"), uri);

			if ( strlen (hostname) > 1 )
				printf (_("COMMAND     : Host: %s\\r\\n\n"), hostname);
			else
				printf (_("COMMAND     : Host: %s\\r\\n\n"), addr);
		/* with HTTP 1.0 */
		} else {
			if ( strlen (hostname) > 1 )
				printf (_("COMMAND     : GET|HEAD http://%s/%s HTTP/1.0\\r\\n\n"), hostname, uri);
			else
				printf (_("COMMAND     : GET|HEAD http://%s/%s HTTP/1.0\\r\\n\n"), addr, uri);
		}

		printf ("              Accept: */*\\r\\n\n");

		if ( httpver == 1 )
			printf ("              Connection: close\\r\\n\n");

		printf ("              User-Agent: %s\\r\\n\\r\\n\n", agent);
		printf ("\n");
	}

	memset (logmsg, 0, 1024);
	/* get header info */
	headlen = http_header (addr, port, ctimeout, rtimeout, rcmd);

	if ( headlen == -1 ) {
		fprintf (stderr, "%s", logmsg);
		return 1;
	}

	if ( verbose ) {
		char xbuf[64] = { 0, };

		print_ansi (GREEN, noansi);
		printf (_("[ Execution ]\n"));
		printf ("-------------------------------------------------------------------------------");
		print_ansi (ENDANSI, noansi);
		printf ("\n");
		printf (_("RETURN CODE : %d\n"), return_code);
		if ( return_length > -1 )
			printf (_("RETURN LENG : %s Bytes\n"), numberFormat (return_length, 0, '.', ',', 0) );
		else
			printf (_("RETURN LENG : Chunked Encoding\n"));
		sprintf ( xbuf, _("Downloading ...  ") );
		printf ( "%s", xbuf );
		rmbuflen += strlen (xbuf);
	}

	if ( return_code != 200 ) {
		if ( verbose ) printf ("\n");
		fprintf (stderr, _("%s return code %d\n"), loginit, return_code);
		if ( verbose ) printf ("\n");
		return 1;
	}

	/* create socket */
	if ( (sock = socket(AF_INET, SOCK_STREAM, 0)) < 1 ) {
		fprintf (stderr, _("%s can't create socket\n"), loginit);
		return 1;
	}

	/* connect http port */
	if ( (retval = http_connect (sock, addr, port, ctimeout)) != 0 ) {
		fprintf (stderr, "%s", logmsg);
		close (sock);
		return retval;
	}

	if ( (retval = write(sock, cmd, strlen(cmd))) < 0 && errno != EAGAIN) {
		fprintf (stderr, _("%s Failed write command in http server\n"), loginit);
		close (sock);
		return -1;
	}

	// waiting for I/O
	FD_ZERO(&w_fds);
	FD_SET(sock, &w_fds);

	if ( select (sock + 1, NULL, &w_fds, NULL, NULL) <= 0) {
		fprintf (stderr, _("%s Failed none write data\n"), loginit);
		close (sock);
		return -1;
	}

	if ( savechk ) {
		if (stat (savefile, &filestat) >= 0 ) {
			fprintf (stderr, _("%s %s is already exists\n"), loginit, savefile);
			close (sock);
			return 1;
		}

		if ( (fd = fopen (savefile, "wb")) == NULL ) {
			fprintf (stderr, _("%s Can't open %s in write mode\n"), loginit, savefile);
			close (sock);
			return 1;
		}
	}

	/* start download */
	start_time = microtime();

	while ( (retval = http_read (sock, rtimeout, buffer)) != 0 )
	{
        if ( sock_code != EAGAIN && sock_code != EINPROGRESS ) {
			fprintf (stderr, _("%s return sock_code %d\n"), loginit, sock_code);
			close (sock);
			return -1;
		}

		if ( retval == -2 ) {
			fprintf (stderr, _("%s Read timeout %s\n"), loginit, strerror (sock_code));
			return -1;
		} else if (retval < 0 ) {
			continue;
		} else {
			if ( ! strncmp (buffer, "HTTP", 4) ) {
				char *_realstart;
				char _realhead[READBUFS] = { 0, };

				strcpy (_realhead, buffer);

				_realstart = strstr (_realhead, "\r\n\r\n");
				_realhead[_realstart - _realhead] = 0;

				headsum = strlen (_realhead) + 4;
				retval -= headsum;
			} else {
				headsum = 0;
			}

			/* print standard out */
			if (savechk && fd != NULL) {
				if ( save_len + retval > SAVEBUF ) {
					//if ( fwrite (buffer + headsum, sizeof (char), retval, fd) != retval ) {
					if ( fwrite (save_value, sizeof (char), save_len, fd) != save_len ) {
						/* close socket */
						close (sock);
						/* close file descritor */
						fclose (fd);

						/* removed gabage file */
						unlink (savefile);

						if ( verbose ) printf ("\n\n");
						fprintf (stderr, _("%s Error writting in file %s\n"), loginit, savefile);
						if ( verbose ) printf ("\n");
						return 1;
					}

					/* init save variables */
					memset (save_value, 0, sizeof (save_value));
					save_len = 0;
				}

				/* saved buffer */
				memmove (save_value + save_len, buffer + headsum, retval);
				save_len += retval;
			}
		}

		filesize += (retval < 0) ? 0 : retval;

		if ( verbose ) {
			switch (mark) {
				case '-' :
					mark = '\\';
					break;
				case '\\' :
					mark = '|';
					break;
				case '|' :
					mark = '/';
					break;
				default :
					mark = '-';
			}
			printf ("%c\b", mark);
		}
	}
	close(sock);

	/* end download */
	end_time = microtime();
	exec_time = end_time - start_time;
	if (exec_time < 0) exec_time = 1;

	if ( savechk && fd != NULL ) {
		if ( save_len > 0 ) {
			if ( fwrite (save_value, sizeof (char), save_len, fd) != save_len ) {
				fclose (fd);
				unlink (savefile);

				if ( verbose ) printf ("\n\n");
				fprintf (stderr, _("%s Error writting in file %s\n"), loginit, savefile);
				if ( verbose ) printf ("\n");

				return 1;
			}
		}
		fclose (fd);
	}

	if ( verbose ) {
		int i = 0;
		for ( i=0; i<rmbuflen; i++ )
			printf ("\b");

		for ( i=0; i<rmbuflen; i++ )
			printf (" ");

		for ( i=0; i<rmbuflen; i++ )
			printf ("\b");

		numberFormat (filesize, 0, '.', ',', 1);
		printf (_(" Bytes Donwload Complete\n\n"));
	}

	if ( filesize > 0 ) {
		bytesecs = filesize / exec_time;
		bitsecs  = (filesize * 8) / exec_time;
		human_size (bytesecs, 1, 0, h_bytesecs);
		human_size (bitsecs, 1, 1, h_bitsecs);
	}

	if ( verbose ) {
		char fh_size[1024];
		human_size (filesize, 1, 0, fh_size);
		print_ansi (GREEN, noansi);
		printf (_("[ Result ]\n"));
		printf ("-------------------------------------------------------------------------------");
		print_ansi (ENDANSI, noansi);
		printf ("\n");
		printf (_("Start Time  : %.2f\n"), start_time);
		printf (_("End   Time  : %.2f\n"), end_time);
		printf (_("Exec  Time  : %.2f\n"), exec_time);
		printf (_("Recv  Size  : %s\n"), fh_size);
		printf ("\n");
	}

	if ( fsize > 0 && fsize != filesize ) {
		fprintf (stderr, _("%s Different file size with given size as -f option\n"), loginit);
		return 1;
	}

	printf ("%s %s %s\n", loginit, h_bitsecs, h_bytesecs);

	if ( verbose ) {
		printf ("\n");
	}

	return 0;
}

void usage (char *prog) {
	fprintf (stderr, _("CHECK-UTILS [%s] v%s : check bandwidth with http download\n"), prog, PACKAGE_VERSION);
	fprintf (stderr, _("     Copyright follows BPL License <http://devel.oops.org/document/bpl>\n"));
	fprintf (stderr, _("     JoungKyun Kim <http://www.oops.org>\n\n"));
	fprintf (stderr, _("Usage : %s [Option[:Value]]\n"), prog);
#ifdef HAVE_GETOPT_LONG
	fprintf (stderr, _("        -A, --agent-name=STR        Agent name\n"));
	fprintf (stderr, _("        -a, --address=ADDRESS       Connected http address without protocol\n"));
	fprintf (stderr, _("        -p, --port=PORT             Connect port number\n"));
	fprintf (stderr, _("        -v, --verbose               print extend result\n"));
	fprintf (stderr, _("        -h, --help                  This (help) message\n"));
	fprintf (stderr, _("        -n, --no-ansi               print without ansi mode\n"));
	fprintf (stderr, _("        -H, --host-header=HOSTNAME  HTTP 1.1 Host Header name\n"));
	fprintf (stderr, _("        -u, --uri=WEBPATH           Uri without first slash\n"));
	fprintf (stderr, _("        -S, --save-file=PATH        save filename\n"));
	fprintf (stderr, _("        -f, --file-size=SIZE        give compare size\n"));
	fprintf (stderr, _("        -c, --conn-timeout=SEC      connect timeout seconds\n"));
	fprintf (stderr, _("        -r, --read-timeout=SEC      read timeout seconds\n"));
#else
	fprintf (stderr, _("        -A STR                      Agent name\n"));
	fprintf (stderr, _("        -a ADDRESS                  Connected http address without protocol\n"));
	fprintf (stderr, _("        -p PORT                     connect port number\n"));
	fprintf (stderr, _("        -v                          print extend result\n"));
	fprintf (stderr, _("        -h                          This (help) message\n"));
	fprintf (stderr, _("        -n                          print without ansi mode\n"));
	fprintf (stderr, _("        -H HOSTNAME                 HTTP 1.1 Host Header name\n"));
	fprintf (stderr, _("        -u WEBPATH                  Uri without first slash\n"));
	fprintf (stderr, _("        -S PATH                     save filename\n"));
	fprintf (stderr, _("        -f SIZE                     give compare size\n"));
	fprintf (stderr, _("        -c SEC                      connect timeout seconds\n"));
	fprintf (stderr, _("        -r SEC                      read timeout seconds\n"));
#endif
	fprintf (stderr, _("        -1                          use HTTP 1.1 [ Default : 1.0 ]\n"));
	fprintf (stderr, _("        -0                          use HTTP 1.0\n"));
}

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
