/*
 * CHECK-UTILS
 * some functions about http protocol
 *
 * CVS ID Tag
 * $Id: http.c,v 1.1 2006/04/10 20:21:48 oops Exp $
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <netinet/tcp.h>
#include <time.h>
#include <checkutils.h>
#include <http.h>

extern int return_code;
extern int sock_code;
extern int return_length;
extern char loginit[1024];
extern char logmsg[1024];

void parse_url (char ** _url, char *str) {
	int _len;
	char __url[256] = { 0, };
	char *_tmp;

	if ( ! strncmp (str, "http://", 7) ) {
		_len = strlen (str) - 7;
		if ( _len > 255 ) {
			strcpy (__url, str + 7);
		} else {
			strncpy (__url, str + 7, 255);
		}
	} else {
		_len = strlen (str);
		if ( _len > 255 ) {
			strcpy (__url, str);
		} else {
			strncpy (__url, str, 255);
		}
	}

	_tmp = (char *) strstr (__url, "/");
	
	if ( _tmp != NULL ) {
		_url[URI]  = (char *) malloc (sizeof (char) * strlen (_tmp) + 1);
		memset (_url[URI], 0, strlen(_tmp) + 1);
		strcpy (_url[URI], _tmp);

		__url[_tmp-__url] = 0;
		_url[HOST] = (char *) malloc (sizeof (char) * strlen (__url) + 1);
		memset (_url[HOST], 0, strlen(__url) + 1);
		strcpy (_url[HOST], __url);
	} else {
		_url[URI]  = (char *) malloc (sizeof (char) * 4);
		_url[HOST] = (char *) malloc (sizeof (char) * strlen (__url) + 1);
		memset (_url[URI], 0, 5);
		memset (_url[HOST], 0, strlen (__url) + 1);
		strcpy (_url[URI], "/");
		strcpy (_url[HOST], __url);
	}
}

int http_connect (int socks, char *dest, int port, int timeout) {
	struct sockaddr_in address;
	struct hostent *hostinfo;
	struct timeval timeouts;
	int    result = 0, sval;
	fd_set r_fds, c_fds;
	socklen_t slen;

	if ( ! (hostinfo = gethostbyname(dest)) ) {
		sprintf(logmsg, _("%s no host\n"), loginit);
		return -1;
	}

	timeouts.tv_sec = TIMEOUT_SEC;
	timeouts.tv_usec = 0;
	setsockopt(socks, SOL_SOCKET, SO_SNDTIMEO, &timeouts, sizeof(timeouts));
	setsockopt(socks, SOL_SOCKET, SO_RCVTIMEO, &timeouts, sizeof(timeouts));
	setsockopt(socks, SOL_SOCKET, SO_KEEPALIVE, NULL, 0);
	setsockopt(socks, SOL_SOCKET, SO_REUSEADDR, NULL, 0);
	setsockopt(socks, IPPROTO_TCP, TCP_NODELAY, NULL, 0);

	address.sin_family = AF_INET;
	address.sin_port = ntohs(port);
	address.sin_addr = *(struct in_addr *) *hostinfo->h_addr_list;

	/* make it nonblocking untion connection */
	if ( nonblock_func (socks, TRUE) == -1 ) {
		sprintf (logmsg, _("%s Failed to convert nonblocking mode\n"), loginit);
		return -1;
	}

	if ( connect (socks, (struct sockaddr *) &address, sizeof (address)) < 0 && errno == EINPROGRESS) {
		/* connected nonblockingly. wait until timeout */
		FD_ZERO (&c_fds);
		FD_ZERO (&r_fds);
		FD_SET (socks, &c_fds);
		FD_SET (socks, &r_fds);
		timeouts.tv_sec = timeout;
		timeouts.tv_usec = 0;

		/* wait until something happens */
		if ( (result = select ( socks + 1, NULL, &c_fds, NULL, &timeouts)) < 1 ) {
			/* failed connect() */
			sprintf (logmsg, _("%s Failed http connection\n"), loginit);
			return -1;
		}

		if ( FD_ISSET (socks, &r_fds) || FD_ISSET (socks, &c_fds) ) {
			if ( FD_ISSET (socks, &r_fds) && FD_ISSET (socks, &c_fds) ) {
				slen = sizeof (sval);

				if ( getsockopt (socks, SOL_SOCKET, SO_ERROR, &sval, &slen) < 0 ) {
					sprintf (logmsg, "%s %s\n", loginit, strerror (ETIMEDOUT));
					close (socks);

					return -1;
				}

				if ( sval != 0 ) {
					sprintf (logmsg, _("%s connection timeout\n"), loginit);
					close (socks);
					return -1;
				}
			}
		}
	}

	return 0;
}

int http_read (int socks, int timeout, char *buf) {
	struct	timeval timeouts;
	int		len = 0;
	int		sel = 0;
	char	rbufs[READBUFS];
	fd_set	r_fds;

	memset (rbufs, '\0', sizeof (rbufs));

	len = recv(socks, rbufs, READBUFS, 0);

	/* return errno */
	sock_code = errno;

	/* waiting for I/O */
	FD_ZERO(&r_fds);
	FD_SET(socks, &r_fds);

	timeouts.tv_sec = timeout;
	timeouts.tv_usec = 0;

	sel = select (socks + 1, &r_fds, NULL, NULL, &timeouts);

	/* case of time out */
	if ( sel == 0 )
		return -2;

	/*
	 * EAGAIN      => 11  Try again
	 * EINPROGRESS => 115 Operation now in progress
	 * ENOENT      => 2   No such file or directory
	 */
	if ( len >= 0 && (errno == EAGAIN || errno == EINPROGRESS || errno == ENOENT) ) {
		if ( sel < 0 )
			return -1;

		if (len < 1) { return 0; }

		memmove (buf, rbufs, len);
	} else len = -1;

	return len;
}

void parse_httpd_header (char *str) {
	const char delimiters[] = "\n";
	char * token, * btoken;
	char headstr[READBUFS], tokstr[128], ret[16], len[128];

	memset (ret, '\0', sizeof(ret));
	memset (len, '\0', sizeof(len));

	strcpy (headstr, str);
	token = (char *) strtok_r (headstr, delimiters, &btoken);

	while ( token != NULL ) {
		memset (tokstr, '\0', sizeof(tokstr));
		strcpy (tokstr, token);

		if ( !strncmp (tokstr, "HTTP/1.1 ", 9) )
			strncpy (ret, tokstr + 9, 3);
		else if ( !strncmp (tokstr, "HTTP/1.0 ", 9) )
			strncpy (ret, tokstr + 9, 3);
		else if ( !strncmp (tokstr, "Content-Length: ", 16) )
			strcpy (len, tokstr + 16);

		token = (char *) strtok_r (NULL, delimiters, &btoken);
	}

	if (! strlen (len) )
		strcpy (len, "-1");

	return_code   = atoi ( trim (ret) );
	return_length = atoi ( trim (len) );
}

long http_header (char * haddr, int port, int ctime, int rtime, char * hcmd) {
	long	sum = 0;
	int		hsock, len = 0, maxsize = 0;
	char	cmd[2048], head[READBUFS], bufs[READBUFS];
	fd_set	hw_fds;

	memset (cmd, '\0', sizeof (cmd));
	memset (bufs, '\0', sizeof (bufs));
	memset (head, '\0', sizeof (head));

	sprintf (cmd, "HEAD %s", hcmd);

	if ( (hsock = socket(AF_INET, SOCK_STREAM, 0)) < 1 ) {
		sprintf (logmsg, _("%s can't create socket\n"), loginit);
		return -1;
	}

	/* connect http port */
	if ( http_connect (hsock, haddr, port, ctime) != 0 ) {
		close (hsock);
		return -1;
	}

	if ( write(hsock, cmd, strlen(cmd)) < 0 && errno != EAGAIN) {
		sprintf (logmsg, _("%s Failed write command in http server\n"), loginit);
		close (hsock);
		return -1;
	}

	// waiting for I/O
	FD_ZERO(&hw_fds);
	FD_SET(hsock, &hw_fds);

	if ( select (hsock + 1, NULL, &hw_fds, NULL, NULL) <= 0) {
		sprintf (logmsg, _("%s Failed none write data\n"), loginit);
		close (hsock);
		return -1;
	}

	while ( (len = http_read (hsock, rtime, bufs)) != 0 ) {
		if ( sock_code != EAGAIN && sock_code != EINPROGRESS ) {
			sprintf (logmsg, _("%s return sock_code %d (%s)\n"), loginit, sock_code, strerror (sock_code));
			close (hsock);
			return -1;
		}

		if ( len == -2 ) {
			sprintf (logmsg, _("%s Read timeout %s\n"), loginit, strerror (sock_code));
			return -1;
		} else if (len < 0 ) {
			continue;
		} else {
			memmove (head + sum, bufs, len);
			sum += len;
		}

		maxsize += len;
		if ( maxsize > 2048 ) {
			sprintf (logmsg, _("%s Header size is too big\n"), loginit);
			close (hsock);
			return -1;
		}
	}

	close (hsock);

	/* get return code and contents length */
	parse_httpd_header (head);

	return sum;
}

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
