/*
 * CHECK-UTILS : httpwatch
 * httpwatch: 
 *    monitor http port with service protocol
 *
 * $Id: httpwatch.c,v 1.5 2006/04/24 01:23:18 oops Exp $
 */

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif
#include <checkutils.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/fcntl.h>
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <ctype.h>
#include <http.h>

void logging (char *str, char *logfile);
void usage (char *prog);

#ifdef HAVE_GETOPT_LONG
static struct option long_options [] = {
	/* Options without arguments: */ 
	{ "help", no_argument, NULL, 'h' },
	{ "no-ansi", no_argument, NULL, 'n' },
	{ "verbose", no_argument, NULL, 'v' },
		    
	/* Options accepting an argument: */
	{ "agent-name", required_argument, NULL, 'a' },
	{ "host-header", required_argument, NULL, 'H' },
	{ "conn-timeout", required_argument, NULL, 'c' },
	{ "read-timeout", required_argument, NULL, 'r' },
	{ "logfile", required_argument, NULL, 'l' },
	{ "port", required_argument, NULL, 'p' }, 
	{ 0, 0, 0, 0 }
};  
#endif


int verbose = 0;
struct tm *loctime;

char loginit[1024];
char logmsg[1024];
int return_code, sock_code;
long return_length, headlen;

int logs = 0;
char logfile[256];

int main(int argc ,char *argv[])
{
	int opt, optchklen = 0;
	time_t now = time (NULL);

	char cmd[2048], rcmd[2048];
	char buffer[READBUFS], timestr[64];
	char *url[2];

	/* option variables */
	int rtimeout = 30, ctimeout = 30, httpver = 0;
	int port = 80, noansi = 0;
	char hostname[256];
	char agent[64];

	/* tmp variabls */
	char _host[256] = { 0, };

	i18n_print();

	memset (hostname, '\0', sizeof (hostname));
	memset (agent, '\0', sizeof (agent));
	memset (buffer, '\0', sizeof (buffer));
	memset (loginit, '\0', sizeof (loginit));
	memset (logfile, '\0', sizeof (logfile));

	loctime = localtime (&now);
	strftime(timestr, sizeof (timestr), "%Y.%m.%d %H:%M:%S", loctime);

#ifdef HAVE_GETOPT_LONG
	while ( (opt = getopt_long (argc, argv, "H:a:c:r:p:l:vnh10?", long_options, (int *) 0)) != EOF ) {
#else
	while ( (opt = getopt (argc, argv, "H:a:c:r:p:l:vnh10?")) != EOF ) {
#endif
		optchklen = 0;

		switch (opt) {
			// Host Header value
			case 'H' :
				if ( ! strncmp (optarg, "http://", 7) )
					optchklen = 7;

				if ( strlen (optarg + optchklen) > 255 ) {
					fprintf (stderr, _("%s Host header is too long\n"), timestr);
					return 1;
				}

				strcpy (hostname, optarg + optchklen);
				break;
			case 'n' :
				noansi = 1;
				break;
			// agent option
			case 'a' :
				if ( strlen (optarg) > 63 ) {
					fprintf (stderr, _("%s Agent name is too long\n"), timestr);
					return 1;
				}

				strcpy (agent, optarg);
				break;
			case 'c' :
				ctimeout = atoi (optarg);
				break;
			case 'r' :
				rtimeout = atoi (optarg);
				break;
			case 'l' :
				strcpy (logfile, optarg);
				logs = 1;
				break;
			case 'p' :
				port = atoi (optarg);
				break;
			case '1' :
				httpver = 1;
				break;
			case '0' :
				break;
			case 'v' :
				verbose = 1;
				break;
			case 'h' :
			default :
				usage (argv[0]);
				return 1;
		}
	}

	if ( (argc - optind) != 1 ) {
		usage (argv[0]);
		return 1;
	}

	if ( strlen (argv[optind]) > 255 ) {
		fprintf (stderr, _("%s URL is too long (max 255)\n"), timestr);
		return 1;
	}

	parse_url (url, argv[optind]);

	if ( strlen (agent) < 1 ) {
		sprintf (agent, "Check-Utils [httpwatch] %s", PACKAGE_VERSION);
	}

	if ( verbose ) {
		setbuf (stdout, NULL);
		printf ("\n");
		print_ansi (BLUE, noansi);
		printf ("CHECK-UTILS [%s] v%s : ", argv[0], PACKAGE_VERSION);
		print_ansi (ENDANSI, noansi);
		printf (_("Monitor http port with service protocol\n\n"));
		printf (_("     Copyright follows BPL License <http://devel.oops.org/document/bpl>\n"));
		printf (_("     JoungKyun Kim <http://www.oops.org>\n\n"));

		print_ansi (GREEN, noansi);
		printf ("[ OPTIONS ]\n");
		printf ("-------------------------------------------------------------------------------");
		print_ansi (ENDANSI, noansi);
		printf ("\n");
		printf ("NOANSI      : %d\n", noansi);
		printf ("HEADER      : %s\n", hostname);
		printf ("ADDRESS     : %s\n", url[HOST]);
		printf ("PORT        : %d\n", port);
		printf ("URI         : %s\n", url[URI]);
		printf ("AGENT       : %s\n", agent);
		printf ("CON_TIMEOUT : %d\n", ctimeout);
		printf ("REV_TIMEOUT : %d\n", rtimeout);
		printf ("LOGGING     : %d\n", logs);
		printf ("\n");
	}

	if ( ctimeout < 1 ) ctimeout = 5;
	if ( rtimeout < 1 ) rtimeout = 5;

	memset (cmd, '\0', sizeof(cmd));
	memset (rcmd, '\0', sizeof(rcmd));

	strcpy (_host, ( strlen (hostname) > 1 ) ? hostname : url[HOST]);
	/* with HTTP 1.1 */
	if ( httpver == 1 ) {
		sprintf (rcmd, "%s HTTP/1.1\r\n"
					   "Host: %s\r\n"
					   "Accept: */*\r\n"
					   "Connection: close\r\n"
					   "User-Agent: %s\r\n\r\n",
				 url[URI], _host, agent);
	/* with HTTP 1.0 */
	} else {
		sprintf (rcmd, "http://%s%s HTTP/1.0\r\n"
					   "Accept: */*\r\n"
					   "User-Agent: %s\r\n\r\n", _host, url[URI], agent);
	}

	sprintf (loginit, _("%s http://%s%s \"Host: %s\""), timestr,
			url[HOST], url[URI], strlen (hostname) > 1 ? hostname : url[HOST]);
	sprintf (cmd, "GET %s", rcmd);

	if ( verbose ) {
		print_ansi (GREEN, noansi);
		printf (_("[ Execution Information ]\n"));
		printf ("-------------------------------------------------------------------------------");
		print_ansi (ENDANSI, noansi);
		printf ("\n");
		printf (_("Connection  : %s\n"), url[HOST]);
		
		/* with HTTP 1.1 */
		if ( httpver == 1 ) {
			printf (_("COMMAND     : HEAD %s HTTP/1.1\\r\\n\n"), url[URI]);

			if ( strlen (hostname) > 1 )
				printf ("              Host: %s\\r\\n\n", hostname);
			else
				printf ("              Host: %s\\r\\n\n", url[HOST]);
		/* with HTTP 1.0 */
		} else {
			if ( strlen (hostname) > 1 )
				printf (_("COMMAND     : HEAD http://%s%s HTTP/1.0\\r\\n\n"), hostname, url[URI]);
			else
				printf (_("COMMAND     : HEAD http://%s%s HTTP/1.0\\r\\n\n"), url[HOST], url[URI]);
		}

		printf ("              Accept: */*\\r\\n\n");

		if ( httpver == 1 )
			printf ("              Connection: close\\r\\n\n");

		printf ("              User-Agent: %s\\r\\n\\r\\n\n", agent);
		printf ("\n");
	}

	memset (logmsg, 0, 1024);
	/* get header info */
	headlen = http_header (url[HOST], port, ctimeout, rtimeout, rcmd);

	if ( headlen == -1 ) {
		if ( verbose )
			fprintf (stderr, "%s", logmsg);

		if ( logs )
			logging (logmsg, logfile);
		return 1;
	}

	if ( verbose ) {
		print_ansi (GREEN, noansi);
		printf (_("[ Execution ]\n"));
		printf ("-------------------------------------------------------------------------------");
		print_ansi (ENDANSI, noansi);
		printf ("\n");
		printf (_("RETURN CODE : %d\n"), return_code);
	}

	sprintf (logmsg, _("%s return code %d\n"), loginit, return_code);

	free (url[URI]);
	free (url[HOST]);

	if ( logs ) logging (logmsg, logfile);

	if ( return_code != 200 ) {
		if ( verbose )
			printf ("\n%s\n", logmsg);
		//if ( logs ) logging (logmsg, logfile);
		return 1;
	}

	return 0;
}

void logging (char *str, char *logfile) {
	FILE *fd = NULL;

	fd = fopen (logfile, "a+");
	if ( fd != NULL ) {
		fwrite (str, sizeof (char), strlen (str), fd);
		fclose (fd);
	}
}

void usage (char *prog) {
	fprintf (stderr, _("CHECK-UTILS [%s] v%s : Monitor http port with service protocol\n"), prog, PACKAGE_VERSION);
	fprintf (stderr, _("     Copyright follows BPL License <http://devel.oops.org/document/bpl>\n"));
	fprintf (stderr, _("     JoungKyun Kim <http://www.oops.org>\n\n"));
	fprintf (stderr, _("Usage : %s [Option[:Value]] http://[URL]\n"), prog);
#ifdef HAVE_GETOPT_LONG
	fprintf (stderr, _("        -a, --agent-name=STR        Agent name\n"));
	fprintf (stderr, _("        -p, --prot=port             port name (default: 80)\n"));
	fprintf (stderr, _("        -n, --no-ansi               print without ansi mode\n"));
	fprintf (stderr, _("        -h, --help                  This (help) message\n"));
	fprintf (stderr, _("        -H, --host-header=HOSTNAME  HTTP 1.1 Host Header name\n"));
	fprintf (stderr, _("        -c, --conn-timeout=SEC      connect timeout seconds\n"));
	fprintf (stderr, _("        -r, --read-timeout=SEC      read timeout seconds\n"));
	fprintf (stderr, _("        -l, --logfile=FILE          logging result at FILE\n"));
	fprintf (stderr, _("        -v, --verbose               print extend result\n"));
#else
	fprintf (stderr, _("        -a STR                      Agent name\n"));
	fprintf (stderr, _("        -p port                     port name (default: 80)\n"));
	fprintf (stderr, _("        -n                          print without ansi mode\n"));
	fprintf (stderr, _("        -h                          This (help) message\n"));
	fprintf (stderr, _("        -H HOSTNAME                 HTTP 1.1 Host Header name\n"));
	fprintf (stderr, _("        -c SEC                      connect timeout seconds\n"));
	fprintf (stderr, _("        -r SEC                      read timeout seconds\n"));
	fprintf (stderr, _("        -l FILE                     logging result at FILE\n"));
	fprintf (stderr, _("        -v                          print extend result\n"));
#endif
	fprintf (stderr, _("        -1                          use HTTP 1.1 [ Default : 1.0 ]\n"));
	fprintf (stderr, _("        -0                          use HTTP 1.0\n"));
}

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
