/* 
 * RTSP header check
 * $Id: rtsp.c,v 1.5 2006/04/24 00:39:01 oops Exp $
 */
#include <checkutils.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#include <sys/types.h>
#include <sys/socket.h>

#ifdef __CYGWIN__
typedef unsigned char uint8_t;
//typedef unsigned int uint32_t;
typedef unsigned long long int uint64_t;
#else
#include <inttypes.h>
#endif

#define BUF_SIZE 2048

#ifdef HAVE_GETOPT_LONG
static struct option long_options [] = {
	/* Options without arguments: */
	{ "help", no_argument, NULL, 'h' },
	{ "verbose", no_argument, NULL, 'v' },

	/* Options accepting an argument: */
	{ "timeout", required_argument, NULL, 't' },
	{ 0, 0, 0, 0 }
};
#endif

int verbose = 0;
int rtspcode = 0;

void usage (char *cmd) {
	fprintf (stderr, _("rtspcheck: rtsp header utility\n"));
	fprintf (stderr, _("    USAGE : %s -[thv] rtsp://URL\n"), cmd);
	fprintf (stderr, _("            -t sec, --timeout=sec connection timeout\n"));
	fprintf (stderr, _("            -h, --help            print this help message\n"));
	fprintf (stderr, _("            -v, --verbose         verbose mode\n"));
}

uint32_t get_flag ( char *data, int offset, int byte ) {
	uint32_t ret;
	int      i;
	int      m;

	m = 8 * (byte - 1);

	ret = data[offset] << m;

	for ( i=1; i<byte; i++ ) {
		m -= 8;
		ret |= data[offset + i] << m;
	}

	return ret;
}

int send_command (int s, char *host, char *uri) {
	char query[1024] = { 0, };
	char enstr[1024] = { 0, };
	char utfstr[1024] = { 0, };

	string_utf_iconv (utfstr, uri, strlen (uri), 8);
	URLEncode (enstr, utfstr);

	sprintf (query, "DESCRIBE rtsp://%s/%s RTSP/1.0\r\n"
					"User-Agent: WMPlayer/9.0.0.2980 guid/3300AD50-2C39-46C0-AE0A-F3ED6E6D7421\r\n"
					"Accept: application/sdp\r\n"
					"Accept-Cahrset: UTF-8, *;q=0.1\r\n"
					"X-Accept-Authentication: Negotiate, NTLM, Digest, Basic\r\n"
					"Accept-Language: ko-KR, *;q=0.1\r\n"
					"CSep: 1\r\n"
					"Supported: com.microsoft.wm.srvppair, com.microsoft.wm.sswitch, "
					"com.microsoft.wm.eosmsg, com.microsoft.wm.predstrm\r\n\r\n",
					host, enstr);

	if ( write (s, query, strlen (query) ) != strlen (query) ) {
		if ( verbose ) fprintf (stderr, _("==> ERROR: write error. maybe closed prot 554.\n"));
		close (s);
		exit (1);
	}

	if ( verbose )
		printf (_("\n**** QUERY :\n\n%s\n\n"), query);

	return strlen (query);
}

int rtsp_responce ( char * data ) {
	char code[4] = { 0, };
	int  ret;

	strncpy ( code, data + 9, 3 );
	ret = atoi (code);

	return ret;
}

int main (int argc, char **argv) {
	int      s;
	char     data[2048] = { 0, };
	int      len;
	char     host[256] = { 0, };
	char    *url, *path, *file, *cp;
	int      opt;
	int      timeout, optchklen;

	timeout = 2;
	verbose = 0;
	rtspcode = 0;

	i18n_print();

#ifdef HAVE_GETOPT_LONG
	while ( (opt = getopt_long (argc, argv, "vt:h", long_options, (int *) 0)) != EOF ) {
#else
	while ( (opt = getopt (argc, argv, "vt:h")) != EOF ) {
#endif
		optchklen = 0;

		switch ( opt ) {
			case 'v' :
				verbose++;
				break;
			case 't' :
				timeout = atoi (optarg);
				break;
			default :
				usage(argv[0]);
				return 1;
		}
	}

	if ( argc - optind < 1 || argc == 1 ) {
		usage (argv[0]);
		return 1;
	}

	url = argv[optind];

	if ( strncmp (url, "rtsp://", 7) ) {
		fprintf (stderr, _("ERROR: invaild rtsp url\n\n"));
		return -1;
	}

	/* parse url */
	strncpy (host, url + 7, 255);
	cp = strchr (host, '/');
	*cp = 0;

	path = strchr (url + 7, '/') + 1;

	if ( verbose ) {
		printf (_("host: >%s<\n"), host);
		printf (_("path: >%s<\n"), path);
	}

	file = strrchr (url, '/');

	/* host connect */
	if ( (s = HOSTconnect (host, timeout, 554, 1)) < 0 ) {
		return -1;
	}

	/* send describe query */
	len = send_command (s, host, path);

	/* get return code */
	memset (data, 0, 2048);
	len = recv (s, data, BUF_SIZE, 0);

	rtspcode = rtsp_responce (data);

	if ( verbose ) {
		printf (_("**** RETURN QYERY:\n\n"));
		if ( verbose < 2 ) {
			char *cx;
			cx = strstr (data, "\r\n\r\n");
			data[cx-data] = 0;
		}

		printf ("%s\n", data);
	} else {
		char *cx;

		cx = strstr (data, "\r\n");
		data[cx-data] = 0;
		printf ("%s\n", data + 9);
	}

	if ( verbose > 1 ) {
		uint32_t ofs = 0xffff;

		len = 1;
		while ( ofs != 0x0d0a && len > 0 ) {
			memset (data, 0, 2048);
			len = recv (s, data, BUF_SIZE, 0);
			printf ("%s", data);
			ofs = get_flag (data, len - 2, 2);
		}
	}

	//printf ("*** %d\n", rtspcode);
	close (s);

	if ( rtspcode > 399 )
		return -1;

	return 0;
}

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
